@@ -56,7 +56,7 @@ from mycli.lexer import MyCliLexer
 from mycli.packages import special
 from mycli.packages.filepaths import dir_path_exists, guess_socket_location
 from mycli.packages.hybrid_redirection import get_redirect_components, is_redirect_command
-from mycli.packages.parseutils import is_destructive, is_dropping_database
+from mycli.packages.parseutils import is_destructive, is_dropping_database, is_valid_connection_scheme
 from mycli.packages.prompt_utils import confirm, confirm_destructive_query
 from mycli.packages.special.favoritequeries import FavoriteQueries
 from mycli.packages.special.main import ArgType
@@ -64,7 +64,7 @@ from mycli.packages.sqlresult import SQLResult
 from mycli.packages.tabular_output import sql_format
 from mycli.packages.toolkit.history import FileHistoryWithTimestamp
 from mycli.sqlcompleter import SQLCompleter
-from mycli.sqlexecute import ERROR_CODE_ACCESS_DENIED, FIELD_TYPES, SQLExecute
+from mycli.sqlexecute import FIELD_TYPES, SQLExecute
 
 try:
     import paramiko
@@ -220,6 +220,10 @@ class MyCli:
         self.prompt_format = prompt or prompt_cnf or c["main"]["prompt"] or self.default_prompt
         self.multiline_continuation_char = c["main"]["prompt_continuation"]
         self.prompt_app = None
+        self.destructive_keywords = [
+            keyword for keyword in c["main"].get("destructive_keywords", "DROP SHUTDOWN DELETE TRUNCATE ALTER UPDATE").split(' ') if keyword
+        ]
+        special.set_destructive_keywords(self.destructive_keywords)
 
     def close(self) -> None:
         if self.sqlexecute is not None:
@@ -346,7 +350,7 @@ class MyCli:
         except IOError as e:
             return [SQLResult(status=str(e))]
 
-        if self.destructive_warning and confirm_destructive_query(query) is False:
+        if self.destructive_warning and confirm_destructive_query(self.destructive_keywords, query) is False:
             message = "Wise choice. Command execution stopped."
             return [SQLResult(status=message)]
 
@@ -460,7 +464,7 @@ class MyCli:
         self,
         database: str | None = "",
         user: str | None = "",
-        passwd: str | None = "",
+        passwd: str | None = None,
         host: str | None = "",
         port: str | int | None = "",
         socket: str | None = "",
@@ -473,6 +477,7 @@ class MyCli:
         ssh_password: str | None = "",
         ssh_key_filename: str | None = "",
         init_command: str | None = "",
+        unbuffered: bool | None = None,
         password_file: str | None = "",
     ) -> None:
         cnf = {
@@ -509,7 +514,7 @@ class MyCli:
                 socket = socket or cnf["socket"] or cnf["default_socket"] or guess_socket_location()
 
         passwd = passwd if isinstance(passwd, str) else cnf["password"]
-        charset = charset or cnf["default-character-set"] or "utf8"
+        charset = charset or self.config["main"].get("default_character_set") or cnf["default-character-set"] or "utf8mb4"
 
         # Favor whichever local_infile option is set.
         use_local_infile = False
@@ -528,10 +533,19 @@ class MyCli:
         # if the passwd is not specified try to set it using the password_file option
         password_from_file = self.get_password_from_file(password_file)
         passwd = passwd if isinstance(passwd, str) else password_from_file
-        passwd = '' if passwd is None else passwd
+
+        # password hierarchy
+        # 1. -p / --pass/--password CLI options
+        # 2. envvar (MYSQL_PWD)
+        # 3. DSN (mysql://user:password)
+        # 4. cnf (.my.cnf / etc)
+        # 5. --password-file CLI option
+
+        # if no password was found from all of the above sources, ask for a password
+        if passwd is None:
+            passwd = click.prompt("Enter password", hide_input=True, show_default=False, default='', type=str, err=True)
 
         # Connect to the database.
-
         def _connect() -> None:
             try:
                 self.sqlexecute = SQLExecute(
@@ -550,33 +564,10 @@ class MyCli:
                     ssh_password,
                     ssh_key_filename,
                     init_command,
+                    unbuffered,
                 )
             except pymysql.OperationalError as e1:
-                if e1.args[0] == ERROR_CODE_ACCESS_DENIED:
-                    if password_from_file is not None:
-                        new_passwd = password_from_file
-                    else:
-                        new_passwd = click.prompt(
-                            f"Password for {user}", hide_input=True, show_default=False, default='', type=str, err=True
-                        )
-                    self.sqlexecute = SQLExecute(
-                        database,
-                        user,
-                        new_passwd,
-                        host,
-                        int_port,
-                        socket,
-                        charset,
-                        use_local_infile,
-                        ssl_config_or_none,
-                        ssh_user,
-                        ssh_host,
-                        int(ssh_port) if ssh_port else None,
-                        ssh_password,
-                        ssh_key_filename,
-                        init_command,
-                    )
-                elif e1.args[0] == HANDSHAKE_ERROR and ssl is not None and ssl.get("mode", None) == "auto":
+                if e1.args[0] == HANDSHAKE_ERROR and ssl is not None and ssl.get("mode", None) == "auto":
                     try:
                         self.sqlexecute = SQLExecute(
                             database,
@@ -594,34 +585,10 @@ class MyCli:
                             ssh_password,
                             ssh_key_filename,
                             init_command,
+                            unbuffered,
                         )
-                    except pymysql.OperationalError as e2:
-                        if e2.args[0] == ERROR_CODE_ACCESS_DENIED:
-                            if password_from_file is not None:
-                                new_passwd = password_from_file
-                            else:
-                                new_passwd = click.prompt(
-                                    f"Password for {user}", hide_input=True, show_default=False, default='', type=str, err=True
-                                )
-                            self.sqlexecute = SQLExecute(
-                                database,
-                                user,
-                                new_passwd,
-                                host,
-                                int_port,
-                                socket,
-                                charset,
-                                use_local_infile,
-                                None,
-                                ssh_user,
-                                ssh_host,
-                                int(ssh_port) if ssh_port else None,
-                                ssh_password,
-                                ssh_key_filename,
-                                init_command,
-                            )
-                        else:
-                            raise e2
+                    except Exception as e2:
+                        raise e2
                 else:
                     raise e1
 
@@ -772,7 +739,7 @@ class MyCli:
         if self.smart_completion:
             self.refresh_completions()
 
-        history_file = os.path.expanduser(os.environ.get("MYCLI_HISTFILE", "~/.mycli-history"))
+        history_file = os.path.expanduser(os.environ.get("MYCLI_HISTFILE", self.config.get("history_file", "~/.mycli-history")))
         if dir_path_exists(history_file):
             history = FileHistoryWithTimestamp(history_file)
         else:
@@ -820,7 +787,10 @@ class MyCli:
             nonlocal mutating
             result_count = 0
             for result in results:
-                title, cur, headers, status = result.get_output()
+                title = result.title
+                cur = result.results
+                headers = result.headers
+                status = result.status
                 command = result.command
                 logger.debug("title: %r", title)
                 logger.debug("headers: %r", headers)
@@ -837,7 +807,7 @@ class MyCli:
                         except ValueError as e:
                             self.echo(f"Invalid watch sleep time provided ({e}).", err=True, fg="red")
                             sys.exit(1)
-                if is_select(status) and cur and cur.rowcount > threshold:
+                if is_select(status) and isinstance(cur, Cursor) and cur.rowcount > threshold:
                     self.echo(
                         f"The result set has more than {threshold} rows.",
                         fg="red",
@@ -887,7 +857,10 @@ class MyCli:
                 if self.show_warnings and isinstance(cur, Cursor) and cur.warning_count > 0:
                     warnings = sqlexecute.run("SHOW WARNINGS")
                     for warning in warnings:
-                        title, cur, headers, status = warning.get_output()
+                        title = warning.title
+                        cur = warning.results
+                        headers = warning.headers
+                        status = warning.status
                         formatted = self.format_output(
                             title,
                             cur,
@@ -971,7 +944,7 @@ class MyCli:
                     return
 
             if self.destructive_warning:
-                destroy = confirm_destructive_query(text)
+                destroy = confirm_destructive_query(self.destructive_keywords, text)
                 if destroy is None:
                     pass  # Query was not destructive. Nothing to do here.
                 elif destroy is True:
@@ -986,10 +959,7 @@ class MyCli:
                 logger.debug("sql: %r", text)
 
                 special.write_tee(self.get_prompt(self.prompt_format) + text)
-                if self.logfile:
-                    self.logfile.write(f"\n# {datetime.now()}\n")
-                    self.logfile.write(text)
-                    self.logfile.write("\n")
+                self.log_query(text)
 
                 successful = False
                 start = time()
@@ -1170,6 +1140,12 @@ class MyCli:
             self.echo(str(e), err=True, fg="red")
             return False
 
+    def log_query(self, query: str) -> None:
+        if isinstance(self.logfile, TextIOWrapper):
+            self.logfile.write(f"\n# {datetime.now()}\n")
+            self.logfile.write(query)
+            self.logfile.write("\n")
+
     def log_output(self, output: str) -> None:
         """Log the output in the audit log, if it's enabled."""
         if isinstance(self.logfile, TextIOWrapper):
@@ -1349,9 +1325,12 @@ class MyCli:
     def run_query(self, query: str, new_line: bool = True) -> None:
         """Runs *query*."""
         assert self.sqlexecute is not None
+        self.log_query(query)
         results = self.sqlexecute.run(query)
         for result in results:
-            title, cur, headers, _status = result.get_output()
+            title = result.title
+            cur = result.results
+            headers = result.headers
             self.main_formatter.query = query
             self.redirect_formatter.query = query
             output = self.format_output(
@@ -1363,13 +1342,16 @@ class MyCli:
                 self.null_string,
             )
             for line in output:
+                self.log_output(line)
                 click.echo(line, nl=new_line)
 
             # get and display warnings if enabled
             if self.show_warnings and isinstance(cur, Cursor) and cur.warning_count > 0:
                 warnings = self.sqlexecute.run("SHOW WARNINGS")
                 for warning in warnings:
-                    title, cur, headers, _status = warning.get_output()
+                    title = warning.title
+                    cur = warning.results
+                    headers = warning.headers
                     output = self.format_output(
                         title,
                         cur,
@@ -1385,7 +1367,7 @@ class MyCli:
         self,
         title: str | None,
         cur: Cursor | list[tuple] | None,
-        headers: list[str] | None,
+        headers: list[str] | str | None,
         expanded: bool = False,
         is_redirected: bool = False,
         null_string: str | None = None,
@@ -1477,8 +1459,16 @@ class MyCli:
 @click.option("-P", "--port", envvar="MYSQL_TCP_PORT", type=int, help="Port number to use for connection. Honors $MYSQL_TCP_PORT.")
 @click.option("-u", "--user", help="User name to connect to the database.")
 @click.option("-S", "--socket", envvar="MYSQL_UNIX_PORT", help="The socket file to use for connection.")
-@click.option("-p", "--password", "password", envvar="MYSQL_PWD", type=str, help="Password to connect to the database.")
-@click.option("--pass", "password", envvar="MYSQL_PWD", type=str, help="Password to connect to the database.")
+@click.option(
+    "-p",
+    "--pass",
+    "--password",
+    "password",
+    is_flag=False,
+    flag_value="MYCLI_ASK_PASSWORD",
+    type=str,
+    help="Prompt for (or enter in cleartext) password to connect to the database.",
+)
 @click.option("--ssh-user", help="User name to connect to ssh server.")
 @click.option("--ssh-host", help="Host name to connect to ssh server.")
 @click.option("--ssh-port", default=22, help="Port to connect to ssh server.")
@@ -1514,6 +1504,7 @@ class MyCli:
 @click.option("-d", "--dsn", default="", envvar="DSN", help="Use DSN configured into the [alias_dsn] section of myclirc file.")
 @click.option("--list-dsn", "list_dsn", is_flag=True, help="list of DSN configured into the [alias_dsn] section of myclirc file.")
 @click.option("--list-ssh-config", "list_ssh_config", is_flag=True, help="list ssh configurations in the ssh config (requires paramiko).")
+@click.option("--ssh-warning-off", is_flag=True, help="Suppress the SSH deprecation notice.")
 @click.option("-R", "--prompt", "prompt", help=f'Prompt format (Default: "{MyCli.default_prompt}").')
 @click.option("-l", "--logfile", type=click.File(mode="a", encoding="utf-8"), help="Log every query and its results to a file.")
 @click.option("--defaults-group-suffix", type=str, help="Read MySQL config groups with the specified suffix.")
@@ -1527,20 +1518,29 @@ class MyCli:
 @click.option(
     "--show-warnings/--no-show-warnings", "show_warnings", is_flag=True, help="Automatically show warnings after executing a SQL statement."
 )
-@click.option("-t", "--table", is_flag=True, help="Display batch output in table format.")
-@click.option("--csv", is_flag=True, help="Display batch output in CSV format.")
+@click.option("-t", "--table", is_flag=True, help="Shorthand for --format=table.")
+@click.option("--csv", is_flag=True, help="Shorthand for --format=csv.")
 @click.option("--warn/--no-warn", default=None, help="Warn before running a destructive query.")
 @click.option("--local-infile", type=bool, help="Enable/disable LOAD DATA LOCAL INFILE.")
 @click.option("-g", "--login-path", type=str, help="Read this path from the login file.")
 @click.option("-e", "--execute", type=str, help="Execute command and quit.")
 @click.option("--init-command", type=str, help="SQL statement to execute after connecting.")
+@click.option(
+    "--unbuffered", is_flag=True, help="Instead of copying every row of data into a buffer, fetch rows as needed, to save memory."
+)
 @click.option("--charset", type=str, help="Character set for MySQL session.")
 @click.option(
     "--password-file", type=click.Path(), help="File or FIFO path containing the password to connect to the db if not specified otherwise."
 )
-@click.argument("database", default="", nargs=1)
+@click.argument("database", default=None, nargs=1)
+@click.option("--noninteractive", is_flag=True, help="Don't prompt during batch input.  Recommended.")
+@click.option(
+    '--format', 'batch_format', type=click.Choice(['default', 'csv', 'tsv', 'table']), help='Format for batch or --execute output.'
+)
+@click.pass_context
 def cli(
-    database: str,
+    ctx: click.Context,
+    database: str | None,
     user: str | None,
     host: str | None,
     port: int | None,
@@ -1580,9 +1580,13 @@ def cli(
     list_ssh_config: bool,
     ssh_config_path: str,
     ssh_config_host: str | None,
+    ssh_warning_off: bool | None,
     init_command: str | None,
+    unbuffered: bool | None,
     charset: str | None,
     password_file: str | None,
+    noninteractive: bool,
+    batch_format: str | None,
 ) -> None:
     """A MySQL terminal client with auto-completion and syntax highlighting.
 
@@ -1593,6 +1597,27 @@ def cli(
       - mycli mysql://my_user@my_host.com:3306/my_database
 
     """
+    # if user passes the --p* flag, ask for the password right away
+    # to reduce lag as much as possible
+    if password == "MYCLI_ASK_PASSWORD":
+        password = click.prompt("Enter password", hide_input=True, show_default=False, default='', type=str, err=True)
+    # if the password value looks like a DSN, treat it as such and
+    # prompt for password
+    elif database is None and password is not None and "://" in password:
+        # check if the scheme is valid. We do not actually have any logic for these, but
+        # it will most usefully catch the case where we erroneously catch someone's
+        # password, and give them an easy error message to follow / report
+        is_valid_scheme, scheme = is_valid_connection_scheme(password)
+        if not is_valid_scheme:
+            click.secho(f"Error: Unknown connection scheme provided for DSN URI ({scheme}://)", err=True, fg="red")
+            sys.exit(1)
+        database = password
+        password = click.prompt("Enter password", hide_input=True, show_default=False, default='', type=str, err=True)
+    # getting the envvar ourselves because the envvar from a click
+    # option cannot be an empty string, but a password can be
+    elif password is None and os.environ.get("MYSQL_PWD") is not None:
+        password = os.environ.get("MYSQL_PWD")
+
     mycli = MyCli(
         prompt=prompt,
         logfile=logfile,
@@ -1604,6 +1629,23 @@ def cli(
         myclirc=myclirc,
     )
 
+    if csv and batch_format not in [None, 'csv']:
+        click.secho("Conflicting --csv and --format arguments.", err=True, fg="red")
+        sys.exit(1)
+
+    if table and batch_format not in [None, 'table']:
+        click.secho("Conflicting --table and --format arguments.", err=True, fg="red")
+        sys.exit(1)
+
+    if not batch_format:
+        batch_format = 'default'
+
+    if csv:
+        batch_format = 'csv'
+
+    if table:
+        batch_format = 'table'
+
     if ssl_enable is not None:
         click.secho(
             "Warning: The --ssl/--no-ssl CLI options are deprecated and will be removed in a future release. "
@@ -1612,6 +1654,15 @@ def cli(
             fg="yellow",
         )
 
+    # ssh_port and ssh_config_path have truthy defaults and are not included
+    if any([ssh_user, ssh_host, ssh_password, ssh_key_filename, list_ssh_config, ssh_config_host]) and not ssh_warning_off:
+        click.secho(
+            "Warning: The built-in SSH functionality is soft deprecated and may be removed in a future release. "
+            "Please discuss or vote on this at https://github.com/dbcli/mycli/issues/1464",
+            err=True,
+            fg="red",
+        )
+
     if list_dsn:
         try:
             alias_dsn = mycli.config["alias_dsn"]
@@ -1629,12 +1680,17 @@ def cli(
         sys.exit(0)
     if list_ssh_config:
         ssh_config = read_ssh_config(ssh_config_path)
-        for host in ssh_config.get_hostnames():
+        try:
+            host_entries = ssh_config.get_hostnames()
+        except KeyError:
+            click.secho('Error reading ssh config', err=True, fg="red")
+            sys.exit(1)
+        for host_entry in host_entries:
             if verbose:
-                host_config = ssh_config.lookup(host)
-                click.secho(f"{host} : {host_config.get('hostname')}")
+                host_config = ssh_config.lookup(host_entry)
+                click.secho(f"{host_entry} : {host_config.get('hostname')}")
             else:
-                click.secho(host)
+                click.secho(host_entry)
         sys.exit(0)
     # Choose which ever one has a valid value.
     database = dbname or database
@@ -1792,6 +1848,7 @@ def cli(
         ssh_password=ssh_password,
         ssh_key_filename=ssh_key_filename,
         init_command=combined_init_cmd,
+        unbuffered=unbuffered,
         charset=charset,
         password_file=password_file,
     )
@@ -1804,15 +1861,20 @@ def cli(
     #  --execute argument
     if execute:
         try:
-            if csv:
-                mycli.main_formatter.format_name = "csv"
-                if execute.endswith(r"\G"):
+            if batch_format == 'csv':
+                mycli.main_formatter.format_name = 'csv'
+                if execute.endswith(r'\G'):
                     execute = execute[:-2]
-            elif table:
-                if execute.endswith(r"\G"):
+            elif batch_format == 'tsv':
+                mycli.main_formatter.format_name = 'tsv'
+                if execute.endswith(r'\G'):
+                    execute = execute[:-2]
+            elif batch_format == 'table':
+                mycli.main_formatter.format_name = 'ascii'
+                if execute.endswith(r'\G'):
                     execute = execute[:-2]
             else:
-                mycli.main_formatter.format_name = "tsv"
+                mycli.main_formatter.format_name = 'tsv'
 
             mycli.run_query(execute)
             sys.exit(0)
@@ -1824,36 +1886,44 @@ def cli(
         mycli.run_cli()
     else:
         stdin = click.get_text_stream("stdin")
-        try:
-            stdin_text = stdin.read()
-        except MemoryError:
-            click.secho("Failed! Ran out of memory.", err=True, fg="red")
-            click.secho("You might want to try the official mysql client.", err=True, fg="red")
-            click.secho("Sorry... :(", err=True, fg="red")
-            sys.exit(1)
-
-        if mycli.destructive_warning and is_destructive(stdin_text):
+        counter = 0
+        for stdin_text in stdin:
+            if counter:
+                if batch_format == 'csv':
+                    mycli.main_formatter.format_name = 'csv-noheader'
+                elif batch_format == 'tsv':
+                    mycli.main_formatter.format_name = 'tsv_noheader'
+                elif batch_format == 'table':
+                    mycli.main_formatter.format_name = 'ascii'
+                else:
+                    mycli.main_formatter.format_name = 'tsv'
+            else:
+                if batch_format == 'csv':
+                    mycli.main_formatter.format_name = 'csv'
+                elif batch_format == 'tsv':
+                    mycli.main_formatter.format_name = 'tsv'
+                elif batch_format == 'table':
+                    mycli.main_formatter.format_name = 'ascii'
+                else:
+                    mycli.main_formatter.format_name = 'tsv'
+            counter += 1
+            warn_confirmed: bool | None = True
+            if not noninteractive and mycli.destructive_warning and is_destructive(mycli.destructive_keywords, stdin_text):
+                try:
+                    # this seems to work, even though we are reading from stdin above
+                    sys.stdin = open("/dev/tty")
+                    # bug: the prompt will not be visible if stdout is redirected
+                    warn_confirmed = confirm_destructive_query(mycli.destructive_keywords, stdin_text)
+                except (IOError, OSError):
+                    mycli.logger.warning("Unable to open TTY as stdin.")
+                    sys.exit(1)
             try:
-                sys.stdin = open("/dev/tty")
-                warn_confirmed = confirm_destructive_query(stdin_text)
-            except (IOError, OSError):
-                mycli.logger.warning("Unable to open TTY as stdin.")
-            if not warn_confirmed:
-                sys.exit(0)
-
-        try:
-            new_line = True
-
-            if csv:
-                mycli.main_formatter.format_name = "csv"
-            elif not table:
-                mycli.main_formatter.format_name = "tsv"
-
-            mycli.run_query(stdin_text, new_line=new_line)
-            sys.exit(0)
-        except Exception as e:
-            click.secho(str(e), err=True, fg="red")
-            sys.exit(1)
+                if warn_confirmed:
+                    mycli.run_query(stdin_text, new_line=True)
+            except Exception as e:
+                click.secho(str(e), err=True, fg="red")
+                sys.exit(1)
+        sys.exit(0)
     mycli.close()
 
 
