@@ -11,7 +11,7 @@ import click
 from click.testing import CliRunner
 from pymysql.err import OperationalError
 
-from mycli.main import MyCli, cli, thanks_picker
+from mycli.main import MyCli, cli, is_valid_connection_scheme, thanks_picker
 import mycli.packages.special
 from mycli.packages.special.main import COMMANDS as SPECIAL_COMMANDS
 from mycli.sqlexecute import ServerInfo, SQLExecute
@@ -40,6 +40,16 @@ CLI_ARGS = [
 ]
 
 
+def test_is_valid_connection_scheme_valid(executor, capsys):
+    is_valid, scheme = is_valid_connection_scheme("mysql://test@localhost:3306/dev")
+    assert is_valid
+
+
+def test_is_valid_connection_scheme_invalid(executor, capsys):
+    is_valid, scheme = is_valid_connection_scheme("nope://test@localhost:3306/dev")
+    assert not is_valid
+
+
 @dbtest
 def test_ssl_mode_on(executor, capsys):
     runner = CliRunner()
@@ -47,7 +57,7 @@ def test_ssl_mode_on(executor, capsys):
     sql = "select * from performance_schema.session_status where variable_name = 'Ssl_cipher'"
     result = runner.invoke(cli, args=CLI_ARGS + ["--csv", "--ssl-mode", ssl_mode], input=sql)
     result_dict = next(csv.DictReader(result.stdout.split("\n")))
-    ssl_cipher = result_dict["VARIABLE_VALUE"]
+    ssl_cipher = result_dict.get("VARIABLE_VALUE", None)
     assert ssl_cipher
 
 
@@ -58,7 +68,7 @@ def test_ssl_mode_auto(executor, capsys):
     sql = "select * from performance_schema.session_status where variable_name = 'Ssl_cipher'"
     result = runner.invoke(cli, args=CLI_ARGS + ["--csv", "--ssl-mode", ssl_mode], input=sql)
     result_dict = next(csv.DictReader(result.stdout.split("\n")))
-    ssl_cipher = result_dict["VARIABLE_VALUE"]
+    ssl_cipher = result_dict.get("VARIABLE_VALUE", None)
     assert ssl_cipher
 
 
@@ -69,7 +79,7 @@ def test_ssl_mode_off(executor, capsys):
     sql = "select * from performance_schema.session_status where variable_name = 'Ssl_cipher'"
     result = runner.invoke(cli, args=CLI_ARGS + ["--csv", "--ssl-mode", ssl_mode], input=sql)
     result_dict = next(csv.DictReader(result.stdout.split("\n")))
-    ssl_cipher = result_dict["VARIABLE_VALUE"]
+    ssl_cipher = result_dict.get("VARIABLE_VALUE", None)
     assert not ssl_cipher
 
 
@@ -80,7 +90,7 @@ def test_ssl_mode_overrides_ssl(executor, capsys):
     sql = "select * from performance_schema.session_status where variable_name = 'Ssl_cipher'"
     result = runner.invoke(cli, args=CLI_ARGS + ["--csv", "--ssl-mode", ssl_mode, "--ssl"], input=sql)
     result_dict = next(csv.DictReader(result.stdout.split("\n")))
-    ssl_cipher = result_dict["VARIABLE_VALUE"]
+    ssl_cipher = result_dict.get("VARIABLE_VALUE", None)
     assert not ssl_cipher
 
 
@@ -91,7 +101,7 @@ def test_ssl_mode_overrides_no_ssl(executor, capsys):
     sql = "select * from performance_schema.session_status where variable_name = 'Ssl_cipher'"
     result = runner.invoke(cli, args=CLI_ARGS + ["--csv", "--ssl-mode", ssl_mode, "--no-ssl"], input=sql)
     result_dict = next(csv.DictReader(result.stdout.split("\n")))
-    ssl_cipher = result_dict["VARIABLE_VALUE"]
+    ssl_cipher = result_dict.get("VARIABLE_VALUE", None)
     assert ssl_cipher
 
 
@@ -878,3 +888,22 @@ def test_global_init_commands(executor):
     expected = "sql_select_limit\t9999\n"
     assert result.exit_code == 0
     assert expected in result.output
+
+
+@dbtest
+def test_execute_with_logfile(executor):
+    """Test that --execute combines with --logfile"""
+    sql = 'select 1'
+    runner = CliRunner()
+
+    with NamedTemporaryFile(mode="w", delete=False) as logfile:
+        result = runner.invoke(mycli.main.cli, args=CLI_ARGS + ["--logfile", logfile.name, "--execute", sql])
+        assert result.exit_code == 0
+
+    assert os.path.getsize(logfile.name) > 0
+
+    try:
+        if os.path.exists(logfile.name):
+            os.remove(logfile.name)
+    except Exception as e:
+        print(f"An error occurred while attempting to delete the file: {e}")
